# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import os
import re
import unittest

from mock import patch

from paste.deploy import loadapp
from paste.deploy.config import PrefixMiddleware

from charmworld.testing import WebTestBase


class TestAppConfig(unittest.TestCase):

    def test_proxy_filter(self):
        ini_file = os.environ.get('INI', 'charmworld.ini')
        ini_path = os.path.normpath(os.path.join(
            os.path.dirname(__file__), '..', '..', '..', ini_file))
        wsgi_app = loadapp('config:%s' % ini_path)
        self.assertTrue(isinstance(wsgi_app, PrefixMiddleware))
        with patch.object(wsgi_app, 'app') as mock_method:
            environ = {
                'METHOD': 'GET',
                'SERVER_NAME': 'example.com',
                'SERVER_PORT': '80',
                'SERVER_PROTOCOL': 'HTTP/1.0',
                'PATH_INFO': '/',
                'HTTP_X_FORWARDED_PROTO': 'https'}
            wsgi_app(environ, None)
            passed_environ = mock_method.call_args[0][0]
        self.assertFalse('HTTP_X_FORWARDED_PROTO' in passed_environ)
        self.assertEqual('https', passed_environ['wsgi.url_scheme'])


class TestBaseViews(WebTestBase):

    def test_homeview(self):
        """Make sure the home view renders correctly."""
        resp = self.app.get(
            '/',
            status=200)

        body = resp.body
        self.assertIn("Juju Charms", body)

        # Make sure our cached_view_config is setting the Cache-control
        # correctly.

        for head, value in resp.headers.iteritems():
            if head == 'Cache-Control':
                self.assertEqual('max-age=450', value)
                break
        else:
            self.fail('Cache-Control header not found.')

    def test_main_template_scheme(self):
        """Image and stylesheet URLs respect the WSGI scheme."""
        src_pattern = re.compile('src="([a-z]+)://')
        css_pattern = re.compile('href="([a-z]+)://.*\.css.*"')
        # Https enabled.
        response = self.app.get(
            '/', extra_environ={'wsgi.url_scheme': 'https'})
        https_count = 0
        for match in src_pattern.finditer(response.ubody):
            https_count += 1
            self.assertEqual('https', match.group(1))
        for match in css_pattern.finditer(response.ubody):
            https_count += 1
            self.assertEqual('https', match.group(1))
        self.assertTrue(https_count >= 2)
        # Http default.
        response = self.app.get('/')
        http_count = 0
        for match in src_pattern.finditer(response.ubody):
            http_count += 1
            self.assertEqual('http', match.group(1))
        for match in css_pattern.finditer(response.ubody):
            http_count += 1
            self.assertEqual('http', match.group(1))
        self.assertTrue(https_count == http_count)
