# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from pyelasticsearch.exceptions import (
    ElasticHttpNotFoundError,
)
from charmworld.models import CharmSource
from charmworld.testing import MongoTestBase
import migrations.migrate
from migrations.migrate import Versions


def fake():
    pass


class MigrationTestBase(MongoTestBase):

    def setUp(self):
        super(MigrationTestBase, self).setUp()
        self.versions = Versions('migrations/versions/')
        true_configure_logging = migrations.migrate.configure_logging
        migrations.migrate.configure_logging = fake
        self.addCleanup(
            setattr, migrations.migrate,
            'comfigure_logging', true_configure_logging)


class TestMigration004(MigrationTestBase):

    def test_migration(self):
        self.use_index_client()
        self.db.create_collection('charm-errors')
        self.versions.run_migration(self.db, self.index_client,
                                    '004_remove_charm_errors.py')
        self.assertNotIn('charm-errors', self.db.collection_names())


class TestMigration008(MigrationTestBase):

    def test_migration(self):
        self.use_index_client()
        source = CharmSource.from_request(self)
        source.save({'_id': 'a', 'icon': 'asdf', 'asdf': 'asdf'})
        source.save({'_id': 'b', 'icon': 'asdf', 'asdf': 'asdf'})
        self.versions.run_migration(self.db, self.index_client,
                                    '008_delete_icon_field.py')
        for charm in source._get_all('a'):
            self.assertNotIn('icon', charm)
        for charm in source._get_all('b'):
            self.assertNotIn('icon', charm)


class TestMigration010(MigrationTestBase):

    def test_migration(self):
        self.use_index_client()
        source = CharmSource.from_request(self)
        source.save({
            '_id': 'a',
            'config': None,
        })
        source.save({
            '_id': 'b',
            'config': {
                'options': {},
            },
        })
        source.save({
            '_id': 'c',
            'config': {
                'options': {'foo': {'default': 'bar'}},
            },
        })
        source.save({'_id': 'd'})
        # This is not acceptable to Elasticsearch, because config is a list.
        self.db.charms.save({
            '_id': 'e',
            'config': ['a'],
        })
        self.versions.get_exodus('010_remap_options.py')(source, source)
        for charm in source._get_all('a'):
            self.assertEqual({
            '_id': 'a',
            'config': None,
        }, charm)
        for charm in source._get_all('b'):
            self.assertEqual({
                '_id': 'b',
                'config': {'options': []},
        }, charm)
        for charm in source._get_all('c'):
            self.assertEqual({
                '_id': 'c',
                'config': {'options': [{
                    'name': 'foo',
                    'default': 'bar',
                }]}
            }, charm)
        for charm in source._get_all('d'):
            self.assertEqual({'_id': 'd'}, charm)
        with self.assertRaises(ElasticHttpNotFoundError):
            self.index_client.get('e')
        self.assertEqual({'_id': 'e', 'config': ['a']},
                         self.db.charms.find_one({'_id': 'e'}))
